module RocketGate
  autoload :VERSION,       'rocketgate/version'

  autoload :Authorization, 'rocketgate/authorization'
  autoload :Configuration, 'rocketgate/configuration'
  autoload :Connection,    'rocketgate/connection'
  autoload :CreditCard,    'rocketgate/credit_card'
  autoload :Customer,      'rocketgate/customer'
  autoload :Request,       'rocketgate/request'
  autoload :Response,      'rocketgate/response'
  autoload :ResponseCode,  'rocketgate/response_code'
  autoload :Transaction,   'rocketgate/transaction'

  autoload :Hashable,      'rocketgate/hashable'
  autoload :Validatable,   'rocketgate/validatable'

  AuthorizationError       = Class.new(StandardError)
  ValidationError          = Class.new(StandardError)
  GatewayRequestError      = Class.new(StandardError)
  GatewayResponseError     = Class.new(StandardError)

  class << self
    def configuration
      @configuration ||= Configuration.new
    end

    def configure
      yield(configuration)
    end

    def connection
      configuration.validate!
      @connection ||= Connection.new(configuration.request_host,
                                     configuration.request_port,
                                     configuration.request_path)
    end

    def reset!
      @configuration, @connection = nil, nil
    end

    def send_request!(request)
      response = connection.make_request!(request.build_gateway_request)

      if response.code.to_i == 200
        return Response.from_xml(response.body)
      else
        raise GatewayRequestError.new("Unexpected status code in gateway response: #{response.code}")
      end
    end
  end
end
