module RocketGate
  class Configuration
    include Validatable

    attr_accessor :currency,
                  :merchant_id,
                  :merchant_password,
                  :request_timeout,
                  :request_host,
                  :request_path,
                  :request_port,
                  :request_version,
                  :require_avs,
                  :require_cvv,
                  :require_scrub,
                  :require_ssl

    validatable :currency, :merchant_id, :merchant_password, :request_host,
                :request_path, :request_port, :request_timeout, :request_version,
                :require_avs, :require_cvv, :require_scrub, :require_ssl

    GATEWAY_LIVE_HOST     = 'gw-16.rocketgate.com'
    GATEWAY_TEST_HOST     = 'dev-gw.rocketgate.com'
    GATEWAY_PORT          = 443
    GATEWAY_SERVLET       = '/gateway/servlet/ServiceDispatcherAccess'
    GATEWAY_VERSION       = 'R1.2'
    DEFAULT_TIMEOUT       = 10
    DEFAULT_CURRENCY      = 'USD'
    USER_AGENT_STRING     = "rocketgate-ruby/#{RocketGate::VERSION}"

    def initialize
      @currency           = DEFAULT_CURRENCY
      @request_host       = GATEWAY_LIVE_HOST
      @request_path       = GATEWAY_SERVLET
      @request_port       = GATEWAY_PORT
      @request_timeout    = DEFAULT_TIMEOUT
      @request_version    = GATEWAY_VERSION
      @require_avs        = true
      @require_cvv        = true
      @require_scrub      = true
      @require_ssl        = true
    end

    def testing!
      @request_host       = GATEWAY_TEST_HOST
      @request_port       = GATEWAY_PORT
      @merchant_id        = '1'
      @merchant_password  = 'testpassword'

      self
    end
  end
end
