module RocketGate
  class CreditCard
    include Hashable, Validatable

    attr_accessor :cvv, :exp_month, :exp_year, :number, :card_hash

    validatable :number, :exp_month, :exp_year

    hashable({
      cardNo:       :number,
      expireMonth:  :exp_month,
      expireYear:   :exp_year,
      cvv2:         :cvv
    })

    def initialize(*args)
      @number, @exp_month, @exp_year, @cvv = *args

      clean_number! if @number
    end

    def self.from_card_hash(card_hash)
      new.tap do |cc|
        cc.card_hash = card_hash
      end
    end

    def to_hash
      card_hash ? { cardHash: card_hash } : super
    end

    def valid?
      !card_hash.nil? || super
    end

    private

    def clean_number!
      @number.delete!("^0-9")
    end
  end
end
