module RocketGate::Hashable
  module ClassMethods
    def hashable(mapping_hash, *mergable_items)
      @mappings, @mergables = {}, []

      mapping_hash.each do |display_name, obj|
        if self.new.send(:is_callable?, obj)
          @mappings[display_name] = obj
        else
          raise RocketGate::ValidationError.new("Invalid attribute: #{obj}")
        end
      end

      mergable_items.each do |obj|
        if self.new.send(:is_callable?, obj)
          @mergables << obj
        else
          raise RocketGate::ValidationError.new("Invalid attribute: #{obj}")
        end
      end
    end

    def mappings
      @mappings ||= {}
    end

    def mergables
      @mergables ||= []
    end
  end

  def self.included(base)
    base.extend(ClassMethods)
  end

  def to_hash
    hash = Hash.new.tap do |hsh|
      self.class.mappings.each do |display_name, obj|
        hsh[display_name] = attribute_or_proc_value(obj)
      end
    end

    self.class.mergables.each do |obj|
      hash.merge!(attribute_or_proc_value(obj))
    end if self.class.mergables.any?

    hash
  end

  private

  def attribute_or_proc_value(obj)
    if obj.is_a?(Proc)
      obj.call
    elsif self.respond_to?(obj)
      self.send(obj)
    end
  end

  def is_callable?(obj)
    obj.is_a?(Proc) || self.respond_to?(obj)
  end
end
