require 'gyoku'

module RocketGate
  class Request
    attr_accessor :transaction

    def initialize(transaction = nil)
      @transaction = transaction
    end

    def build_gateway_request
      transaction.validate!

      parameters = base_parameters
      parameters.merge!(transaction.to_hash)

      xml_head = '<?xml version="1.0" encoding="UTF-8"?>'
      xml_body = Gyoku.xml({
        gatewayRequest: parameters.select{|_,v| !v.nil? }
      })

      [xml_head, xml_body].join
    end

    def base_parameters
      config = RocketGate.configuration
      config.validate!

      base_params = {
        version:          config.request_version,
        merchantID:       config.merchant_id,
        merchantPassword: config.merchant_password
      }
      verification_params = {}

      unless transaction.nil?
        verification_params[:cvv2Check] = yes_or_ignore(config.require_cvv) unless transaction.is_hashed?
        verification_params[:avsCheck]  = yes_or_ignore(config.require_avs)
        verification_params[:scrub]     = yes_or_ignore(config.require_scrub)
        base_params.merge!(verification_params) unless transaction.is_referenced?
      end

      base_params
    end

    def yes_or_ignore(boolean)
      boolean ? 'YES' : 'IGNORE'
    end
  end
end
