module RocketGate::Validatable
  module ClassMethods
    def validatable(*args)
      args.each do |attribute|
        if self.new.respond_to?(attribute)
          validatable_attributes << attribute
        else
          raise RocketGate::ValidationError.new("Invalid attribute: #{attribute}")
        end
      end
    end

    def validatable_attributes
      @validatable_attributes ||= []
    end
  end

  def self.included(base)
    base.extend(ClassMethods)
  end

  def valid?
    self.class.validatable_attributes.all? {|v| is_object_valid?(send(v)) }
  end

  def validate!
    if valid?
      self
    else
      message = "Invalid attributes: #{invalid_attributes.join(', ')}"
      raise RocketGate::ValidationError.new(message) unless valid?
    end
  end

  def invalid_attributes
    self.class.validatable_attributes.select {|v| !is_object_valid?(send(v)) }
  end

  private

  def is_class_validatable?(klazz)
    klazz.included_modules.include?(RocketGate::Validatable)
  end

  def is_object_valid?(obj)
    if is_class_validatable?(obj.class)
      obj.valid?
    else
      !obj.nil?
    end
  end
end
